@@ -1,8 +1,12 @@
-import threading
-import api
-import keep
-import config
+from __future__ import absolute_import
+
+from builtins import object
 import copy
+import threading
+
+import arvados
+import arvados.keep as keep
+import arvados.config as config
 
 class ThreadSafeApiCache(object):
     """Threadsafe wrapper for API objects.
@@ -21,7 +25,7 @@ class ThreadSafeApiCache(object):
 
     def localapi(self):
         if 'api' not in self.local.__dict__:
-            self.local.api = api.api_from_config('v1', apiconfig=self.apiconfig)
+            self.local.api = arvados.api_from_config('v1', apiconfig=self.apiconfig)
         return self.local.api
 
     def __getattr__(self, name):
