@@ -1,5 +1,8 @@
+from __future__ import absolute_import
+from builtins import range
+from builtins import object
 import arvados
-import arvados_testutil as tutil
+from . import arvados_testutil as tutil
 
 class ManifestExamples(object):
     def make_manifest(self,
